estimate <- function(object, ...)
  UseMethod("estimate", object, ...)

estimate.lm <- function(object, ...) {
  b <- coef(object)
  if(m <- match("(Intercept)", names(b), nomatch = 0)) {
    l <- length(b) - 1
    if(l > 0) names(b)[ - m] <- paste("b", 1:l, sep = "")
    names(b)[m] <- "b0"
  } else {
    names(b) <- paste("b", seq(along = b), sep = "")
  }
  V <- vcov(object)
  dimnames(V) <- list(names(b), names(b))
  estimate <- list(parameter = b, variance = V)
  class(estimate) <- "estimate"
  estimate
}

estimate.nls <- function(object, ...)
  structure(list(parameter = coef(object),
                 variance = vcov(object)), class = "estimate")

as.estimate <- function(x)
  if(inherits(x, "estimate")) x else estimate(x)

is.estimate <- function(x) inherits(x, "estimate")

print.estimate <- function(x, ...) {
  print(cbind(Parameters = x$parameter,
              SE = sqrt(diag(x$variance))))
  invisible(x)
}

estimate.estimate <-
  function(object, transforms, data = sys.parent(),
           se.only = F, ...)
{
  getval <- function(x, pframe, data)
    eval(do.call("substitute", list(x[[length(x)]], pframe)), data)

  getblock <- function(transform, pframe, data) {
    getcol <- function(x, fun, pframe, data)
      eval(do.call("substitute", list(D(fun, x), pframe)), data)

    fun <- transform[[length(transform)]]
    do.call("cbind", lapply(names(pframe), getcol,
                            fun = fun, pframe = pframe, data = data))
  }

  pframe <- as.list(object$parameter)
  if(!(exists("is.R") && is.R()) && version$major <= 4)
    names(pframe) <- names(object$parameter)
  if(inherits(transforms, "formula"))
    transforms <- list(transforms)
  tnames <- unlist(lapply(transforms, function(x)
                          if(length(x) == 3) x[[2]] else ""))
  tvalues <-
    lapply(transforms, getval, pframe = pframe, data = data)
  names(tvalues) <- tnames
  tvalues <- unlist(tvalues)

  X <- do.call("rbind", lapply(transforms, getblock,
                               pframe = pframe, data = data))
  XV <- X %*% object$variance
  if(se.only) {
    se <- as.vector(sqrt((XV * X) %*% rep(1, ncol(X))))
    data.frame(Parameters = tvalues, SE = se)
  } else {
    structure(list(parameter = tvalues,
                   variance = XV %*% t(X)),
              class = "estimate")
  }
}

